
# Copyright (C) 2020 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Earthquake json file generator.

This script generates earthquake json files using notices from the
internet and USGS PDL client.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""

import os, sys, glob, optparse, warnings, time, json

if not os.getenv("DISPLAY", None):
    import matplotlib
    matplotlib.use("agg", warn=False)

from datetime import datetime
import numpy as np
import subprocess
from subprocess import Popen
from lxml import etree

#import lal.gpstime
import astropy.time

from seismon import (eqmon, utils)

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def parse_commandline():
    """@parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-p", "--paramsFile", help="Seismon params file.",
                      default ="/home/mcoughlin/Seismon/seismon/input/seismon_params_traveltimes.txt")
    parser.add_option("-c", "--paramsFileCopy", help="Seismon copy params file.",
                      default ="/home/mcoughlin/Seismon/seismon/input/seismon_params_traveltimes_copy.txt")

    parser.add_option("-t", "--publicFileType", help="Type of public data.",
                      default ="hour")
    parser.add_option("-s", "--gpsStart", help="GPS Start Time.", default=1000000000,type=int)
    parser.add_option("-e", "--gpsEnd", help="GPS End Time.", default=1200000000,type=int)
    parser.add_option("-m", "--minMagnitude", help="Minimum earthquake magnitude.", default=5.0,type=float)
    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    parser.add_option("--doPublic",  action="store_true", default=False)
    parser.add_option("--doPrivate",  action="store_true", default=False)
    parser.add_option("--doDatabase",  action="store_true", default=False)
    parser.add_option("--doIRIS",  action="store_true", default=False)
    parser.add_option("--doFake",  action="store_true", default=False)
    parser.add_option("--doMultipleEvents",  action="store_true", default=False)
    parser.add_option("--doCopy",  action="store_true", default=False)
    parser.add_option("--doPurge",  action="store_true", default=False)
    parser.add_option("--doMoment",  action="store_true", default=False)
    parser.add_option("--doCMT",  action="store_true", default=False)

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running network_eqmon..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@create params structure

    @param opts
        command line options
    """

    params = utils.readParamsFromFile(opts.paramsFile)
    params["publicFileType"] = opts.publicFileType
    params["gpsStart"] = opts.gpsStart
    params["gpsEnd"] = opts.gpsEnd
    params["minMagnitude"] = opts.minMagnitude

    params["doPublic"] = opts.doPublic
    params["doPrivate"] = opts.doPrivate
    params["doDatabase"] = opts.doDatabase
    params["doIRIS"] = opts.doIRIS
    params["doFake"] = opts.doFake
    params["doMultipleEvents"] = opts.doMultipleEvents
    params["doCopy"] = opts.doCopy
    params["doPurge"] = opts.doPurge
    params["doMoment"] = opts.doMoment
    params["doCMT"] = opts.doCMT

    params["paramsFile"] = opts.paramsFile
    params["paramsFileCopy"] = opts.paramsFileCopy

    return params

class NumpyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

def write_file(file,tree):
    """@write eqmon file

    @param file
        eqmon file
    @param tree
        xml tree
    """

    f = open(file,'w+')
    f.write('%s'%tree)
    f.close()

def write_info(file,attributeDic):
    """@write eqmon file

    @param file
        eqmon file
    @param attributeDic
        eqmon structure
    """

    root = etree.Element('eqmon')
    for key, value in attributeDic.items():
        if not key == "traveltimes":
            element = etree.SubElement(root,key)
            element.text = str(value)
    if "traveltimes" in attributeDic:
        element = etree.SubElement(root,'traveltimes')
        for key, value in attributeDic["traveltimes"].items():
            subelement = etree.SubElement(element,key)
            for category in value:
                if category in ["Latitudes","Longitudes","arrivalMin","arrivalMax"]:
                    continue
                subsubelement = etree.SubElement(subelement,category)
                subsubelement.text = write_array(value[category])

    tree = etree.tostring(root, pretty_print=True).decode()
    write_file(file,tree)
    #tree = etree.ElementTree(root)
    #tree.write(file, pretty_print=True, xml_declaration=True)

def write_array(array):
    """@write array

    @param array
        write array of value
    """
    if len(array) == 1:
        array = np.array(array).flatten()
        text = str(array[0])
    else:
        text = ' '.join([str(x) for x in array])
    return text

def write_context(path):
    """@write context file

    @param path
        path to earthquake directory
    """

    tm = time.struct_time(time.gmtime())
    dt = datetime.fromtimestamp(time.mktime(tm))
    currentGPS = astropy.time.Time(dt, format='datetime', scale='utc').gps

    files = glob.glob(os.path.join(path,"*.xml"))

    f = open(os.path.join(path,"context.txt"),"w")
    for file in files:
        fileSplit = file.split("/")
        fileName = fileSplit[-1]
        if fileName == "context.txt":
            continue

        fileNameSplit = fileName.replace(".xml","").split("-")
        fileGPS = int(fileNameSplit[-1])

        if fileGPS < currentGPS - 12*3600:
            continue

        f.write("%s\n"%fileName)
    f.close()

def delete_files(path,gps):
    """@delete eqmon files

    @param path
        path to eqmon files
    @param gps
        gps time before which to delete files
    """

    files = glob.glob(os.path.join(path,"*.xml"))

    for file in files:
        fileSplit = file.split("/")
        fileName = fileSplit[-1]
        if fileName == "context.txt":
            continue
        fileNameSplit = fileName.replace(".xml","").split("-")
        fileGPS = int(fileNameSplit[-1])
        
        if fileGPS < gps:
            os.system("rm %s"%file)

def readParamsFromFileCopy(file):
    """@read seismon params file

    @param file
        seismon params file
    """

    params = {}
    if os.path.isfile(file):
        with open(file,'r') as f:
            for line in f:
                line_without_return = line.split("\n")
                line_split = line_without_return[0].split(",")
                params[line_split[0]] = line_split[1].replace('"','')
    return params

def rsync_files(params):
    """@rsync eqmon files

    @param path
        path to rsync 
    """

    import pexpect

    paramsCopy = readParamsFromFileCopy(params["paramsFileCopy"])

    user = paramsCopy["user"]
    password = paramsCopy["password"]
    host = paramsCopy["host"]
    path = paramsCopy["path"]

    rsyncCommand = 'rsync -avz --delete %s %s:%s' % (params["eventfilesLocation"],host,path)
    child = pexpect.spawn(rsyncCommand)
    i = child.expect("id_rsa':",timeout=20)
    child.sendline(password)
    data = child.read()
    child.close()

def download_publiceventfiles(params):
    """@download events from usgs website

    @param params
        seismon params structure
    """

    path = params["publicdataLocation"]

    if params["publicFileType"] == "hour":
        file = "https://earthquake.usgs.gov/earthquakes/feed/v1.0/summary/all_hour.geojson"
    elif params["publicFileType"] == "month":
        file = "https://earthquake.usgs.gov/earthquakes/feed/v1.0/summary/4.5_month.geojson"
    elif params["publicFileType"] == "day":
        file = "https://earthquake.usgs.gov/earthquakes/feed/v1.0/summary/all_day.geojson"
    else:
        error("Public file type not supported")

    x = subprocess.Popen(["wget",file,"-O","%s/events.txt"%path])
    x.wait()
    if x.returncode==0:
        print('Copying complete')
    else:
        print('Copying failed')

def save_fake_event(params,timeFolder,eventName):

    attributeDic = []
    eqmonfile = os.path.join(timeFolder,"eqmon.xml")
    quakemlfile = os.path.join(timeFolder,"quakeml.xml")

    #if os.path.isfile(os.path.join(timeFolder,"eqxml.txt")):
    #    return

    f = open(os.path.join(timeFolder,"eqxml.txt"),"w")
    f.write("Done")
    f.close()

    attributeDic = eqmon.read_eqmon([],eqmonfile)

    if attributeDic == []:
        return

    if (not "GPS" in attributeDic) or (not "Magnitude" in attributeDic):
        return

    if params["doMultipleEvents"]:
        file = os.path.join(params["eventfilesLocation"],"fake/%s-%.0f-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"],attributeDic["SentGPS"]))
    else:
        file = os.path.join(params["eventfilesLocation"],"fake/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))

    if attributeDic["Magnitude"] >= float(params["minMagnitude"]):
        write_info(file,attributeDic)

        print("%s added at "%attributeDic["eventName"], time.time(), ". %.3f seconds after event"%(attributeDic["SentGPS"] - attributeDic["GPS"]))

def save_moment_event(params,timeFolder,eventName):

    attributeDic = []
    productfile = os.path.join(timeFolder,"quakeml.xml")

    if os.path.isfile(os.path.join(timeFolder,"eqxml.txt")):
        return

    #f = open(os.path.join(timeFolder,"eqxml.txt"),"w")
    #f.write("Done")
    #f.close()

    if os.path.isfile(productfile):
        attributeDic = eqmon.read_product(productfile,eventName)

    if attributeDic == []:
        return

    if params["doMultipleEvents"]:
        file = os.path.join(params["eventfilesLocation"],"moment/%s-%.0f-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"],attributeDic["SentGPS"]))
    else:
        file = os.path.join(params["eventfilesLocation"],"moment/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))

    if attributeDic["Magnitude"] >= float(params["minMagnitude"]):
        write_info(file,attributeDic)

        print("%s added at "%attributeDic["eventName"], time.time(), ". %.3f seconds after event"%(attributeDic["SentGPS"] - attributeDic["GPS"]))

def save_private_event(params,timeFolder,eventName):

    attributeDic = []
    eqxmlfile = os.path.join(timeFolder,"eqxml.xml")
    quakemlfile = os.path.join(timeFolder,"quakeml.xml")

    #if os.path.isfile(os.path.join(timeFolder,"eqxml.txt")):
    #    return

    f = open(os.path.join(timeFolder,"eqxml.txt"),"w")
    f.write("Done")
    f.close()

    if os.path.isfile(eqxmlfile):
        attributeDic = eqmon.read_eqxml(eqxmlfile,eventName)
    elif os.path.isfile(quakemlfile):
        attributeDic = eqmon.read_quakeml(quakemlfile,eventName)

    if attributeDic == []:
        return

    if (not "GPS" in attributeDic) or (not "Magnitude" in attributeDic):
        return

    if params["doMultipleEvents"]:
        #filename = os.path.join(params["eventfilesLocation"],"private/%s-%.0f-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"],attributeDic["SentGPS"]))
        filename = os.path.join(params["eventfilesLocation"],"private/%s-%.0f-%.0f.json"%(attributeDic["eventName"],attributeDic["GPS"],attributeDic["SentGPS"]))
    else:
        #filename = os.path.join(params["eventfilesLocation"],"private/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))
        filename = os.path.join(params["eventfilesLocation"],"private/%s-%.0f.json"%(attributeDic["eventName"],attributeDic["GPS"]))

    if (attributeDic["GPS"] < params["gpsStart"]) or \
       (attributeDic["GPS"] > params["gpsEnd"]):
        return

    if attributeDic["Magnitude"] >= float(params["minMagnitude"]):
        #write_info(filename, attributeDic)
        with open(filename, 'w') as json_file:
            json.dump(attributeDic, json_file, cls=NumpyEncoder)

        print("%s added at "%attributeDic["eventName"], time.time(), ". %.3f seconds after event"%(attributeDic["SentGPS"] - attributeDic["GPS"]))

def cmt_events(params):
    """@write pdl events

    @param params
        seismon params structure
    """

    numEventsAdded = 0

    years = np.arange(2007,2018)
    years = np.arange(2017,2018)
    months = ["jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"]

    import obspy

    for year in years:
        for month in months:
            try:
                catfile = "http://www.ldeo.columbia.edu/~gcmt/projects/CMT/catalog/NEW_MONTHLY/%d/%s%d.ndk"%(year,month,np.mod(year,100))
                cat = obspy.readEvents(catfile)
                print(catfile)
            except:
                continue 

            for event in cat:
                attributeDic = eqmon.cmtread(event)

                print(attributeDic["GPS"], attributeDic["Magnitude"], attributeDic["Latitude"],attributeDic["Longitude"])

                if not "GPS" in attributeDic:
                    continue
                if os.path.isfile(os.path.join(params["eventfilesLocation"],"cmt/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))):
                    continue

                file = os.path.join(params["eventfilesLocation"],"cmt/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))
                if attributeDic["Magnitude"] >= float(params["minMagnitude"]):
                    write_info(file,attributeDic)

                    print("%s added at "%attributeDic["eventName"], time.time(), ". %.3f seconds after event"%(attributeDic["SentGPS"] - attributeDic["GPS"]))
                    numEventsAdded = numEventsAdded + 1

    return numEventsAdded

def moment_events(params):
    """@write pdl events

    @param params
        seismon params structure
    """

    numEventsAdded = 0

    folders = glob.glob(os.path.join(params["momentDataLocation"],"*"))
    #folders = []
    for numFolder in range(len(folders)):

        folder = folders[numFolder]

        folderSplit = folder.split("/")
        eventName = folderSplit[len(folderSplit) - 1]

        dataFolder = os.path.join(params["momentDataLocation"],eventName,eventName[0:2])
        timeFolders = glob.glob(os.path.join(dataFolder,"*"))
        timeFolders = sorted(timeFolders)

        if timeFolders == []:
            continue

        if params["doMultipleEvents"]:
            if len(timeFolders) == 1:
                continue

        for timeFolder in timeFolders:
                save_moment_event(params,timeFolder,eventName)
                numEventsAdded = numEventsAdded + 1

    return numEventsAdded

def private_events(params):
    """@write pdl events

    @param params
        seismon params structure
    """

    numEventsAdded = 0

    if params["doPurge"]:
        sys_command = "find %s/* -type d -mtime +7 -exec rm -rf {} \;"%params["dataLocation"]
        os.system(sys_command)
        sys_command = "find %s/* -type d -mtime +7 -exec rm -rf {} \;"%params["internalDataLocation"]
        os.system(sys_command)

    folders = glob.glob(os.path.join(params["dataLocation"],"*"))
    #folders = []
    for numFolder in range(len(folders)):

        folder = folders[numFolder]

        folderSplit = folder.split("/")
        eventName = folderSplit[len(folderSplit) - 1]
        dataFolder = os.path.join(params["dataLocation"],eventName,eventName[0:2])
        timeFolders = glob.glob(os.path.join(dataFolder,"*"))
        timeFolders = sorted(timeFolders)

        if timeFolders == []:
            continue

        if params["doMultipleEvents"]:
            if len(timeFolders) == 1:
                continue

        for timeFolder in timeFolders:
                save_private_event(params,timeFolder,eventName)
                numEventsAdded = numEventsAdded + 1

    folders = glob.glob(os.path.join(params["internalDataLocation"],"*"))
    for numFolder in range(len(folders)):

        folder = folders[numFolder]

        folderSplit = folder.split("/")
        eventName = folderSplit[len(folderSplit) - 1]
        dataFolder = os.path.join(params["internalDataLocation"],eventName,eventName[0:2])
        timeFolders = glob.glob(os.path.join(dataFolder,"*"))
        timeFolders = sorted(timeFolders)

        if timeFolders == []:
            continue

        if params["doMultipleEvents"]:
            if len(timeFolders) == 1:
                continue

        for timeFolder in timeFolders:
                save_private_event(params,timeFolder,eventName)
                numEventsAdded = numEventsAdded + 1

    path = os.path.join(params["eventfilesLocation"],"private")
    write_context(path)

    return numEventsAdded

def public_events(params):
    """@write usgs website events

    @param params
        seismon params structure
    """

    numEventsAdded = 0

    download_publiceventfiles(params)
    events_text = open(os.path.join(params["publicdataLocation"],"events.txt"),"r").read()
    events = json.loads(events_text)

    for event in events["features"]:
        attributeDic = eqmon.jsonread(event)

        if not "GPS" in attributeDic:
            continue
        if os.path.isfile(os.path.join(params["eventfilesLocation"],"public/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))):
            continue

        if (attributeDic["GPS"] < params["gpsStart"]) or \
           (attributeDic["GPS"] > params["gpsEnd"]):
            continue

        #filename = os.path.join(params["eventfilesLocation"],"public/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))
        filename = os.path.join(params["eventfilesLocation"],"public/%s-%.0f.json"%(attributeDic["eventName"],attributeDic["GPS"]))
        if attributeDic["Magnitude"] >= float(params["minMagnitude"]):
            #write_info(filename,attributeDic)
            with open(filename, 'w') as json_file: 
                json.dump(attributeDic, json_file, cls=NumpyEncoder)

            print("%s added at "%attributeDic["eventName"], time.time(), ". %.3f seconds after event"%(attributeDic["SentGPS"] - attributeDic["GPS"]))
            numEventsAdded = numEventsAdded + 1

    return numEventsAdded

def database_events(params):
    """@write usgs database events

    @param params
        seismon params structure
    """

    numEventsAdded = 0

    with open(os.path.join(params["databasedataLocation"],"events.txt")) as f:

        for line in f:

            event = line.replace("\n","")
            attributeDic = eqmon.databaseread(event)

            if not "GPS" in attributeDic:
                continue
            if os.path.isfile(os.path.join(params["eventfilesLocation"],"database/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))):
                continue

            file = os.path.join(params["eventfilesLocation"],"database/%s-%.0f.xml"%(attributeDic["eventName"],attributeDic["GPS"]))
            if attributeDic["Magnitude"] >= float(params["minMagnitude"]):
                write_info(file,attributeDic)

            print("%s added at "%attributeDic["eventName"], time.time(), ". %.3f seconds after event"%(attributeDic["SentGPS"] - attributeDic["GPS"]))
            numEventsAdded = numEventsAdded + 1

    return numEventsAdded

def iris_events(params):
    """@write usgs database events

    @param params
        seismon params structure
    """

    import obspy.clients.fdsn, obspy.core

    #starttime = lal.gpstime.gps_to_utc(params["gpsStart"])
    #endtime = lal.gpstime.gps_to_utc(params["gpsEnd"])
    #starttime = obspy.core.UTCDateTime(starttime)
    #endtime = obspy.core.UTCDateTime(endtime)

    starttime = astropy.time.Time(params["gpsStart"], format='gps')
    endtime = astropy.time.Time(params["gpsEnd"], format='gps')
    starttime = obspy.core.UTCDateTime(starttime.utc.iso)
    endtime = obspy.core.UTCDateTime(endtime.utc.iso)

    client = obspy.clients.fdsn.Client("IRIS")
    #client = obspy.fdsn.client.Client("http://earthquake.usgs.gov")
    
    events = client.get_events(minmagnitude=params["minMagnitude"],starttime=starttime,endtime=endtime)

    ifos = ["LHO","LLO","GEO","VIRGO","Homestake","FortyMeter","MIT"]

    numEventsAdded = 0
    for event in events.events:
        attributeDic = eqmon.irisread(event)

        print(attributeDic["GPS"], attributeDic["Magnitude"], attributeDic["Latitude"],attributeDic["Longitude"])

        for ifo in ifos:
            attributeDicTemp = eqmon.eqmon_loc(attributeDic,ifo)
            traveltimes = attributeDicTemp["traveltimes"][ifo]
            print("%s %.0f %.0f %.0f %.0f %.0f %.5e %.0f"%(ifo,max(traveltimes["Ptimes"]),max(traveltimes["Stimes"]),max(traveltimes["Rtwotimes"]),max(traveltimes["RthreePointFivetimes"]),max(traveltimes["Rfivetimes"]),traveltimes["Rfamp"][0],max(traveltimes["Distances"])))
        print("")
        print("")

        if not "GPS" in attributeDic:
            continue
        filename = os.path.join(params["eventfilesLocation"],"iris/%s-%.0f.json"%(attributeDic["eventName"],attributeDic["GPS"]))
        if os.path.isfile(filename):
            continue

        if (attributeDic["GPS"] < params["gpsStart"]) or \
           (attributeDic["GPS"] > params["gpsEnd"]):
            continue

        if attributeDic["Magnitude"] >= float(params["minMagnitude"]):
            #write_info(filename,attributeDic)
            with open(filename, 'w') as json_file:
                json.dump(attributeDic, json_file, cls=NumpyEncoder)

            print("%s added at "%attributeDic["eventName"], time.time(), ". %.3f seconds after event"%(attributeDic["SentGPS"] - attributeDic["GPS"]))
            numEventsAdded = numEventsAdded + 1

    return numEventsAdded

def fake_events(params):
    """@write fake events

    @param params
        seismon params structure
    """

    numEventsAdded = 0
    folders = glob.glob(os.path.join(params["eventfilesLocation"],"fakepdl/*"))
    #folders = []
    for numFolder in range(len(folders)):

        folder = folders[numFolder]

        folderSplit = folder.split("/")
        eventName = folderSplit[len(folderSplit) - 1]

        dataFolder = os.path.join(params["eventfilesLocation"],"fakepdl",eventName,eventName[0:2])
        timeFolders = glob.glob(os.path.join(dataFolder,"*"))
        timeFolders = sorted(timeFolders)

        if timeFolders == []:
            continue

        if params["doMultipleEvents"]:
            if len(timeFolders) == 1:
                continue

        for timeFolder in timeFolders:
                save_fake_event(params,timeFolder,eventName)
                numEventsAdded = numEventsAdded + 1
        
    return numEventsAdded


def run_traveltimes():
    """@run traveltime calculator
    """

    warnings.filterwarnings("ignore")

    # Parse command line
    opts = parse_commandline()
    # Parse command line
    params = params_struct(opts)

    if params["doPublic"]:
        print("Running public events...")
        numEventsAddedPublic = public_events(params)
    if params["doPrivate"]:
        print("Running private events...")
        numEventsAddedPrivate = private_events(params)
    if params["doDatabase"]:
        print("Running database events...")
        numEventsAddedDatabase = database_events(params)
    if params["doIRIS"]:
        print("Running IRIS events...")
        numEventsIRIS = iris_events(params)
    if params["doFake"]:
        print("Running Fake events...")
        numEventsFake = fake_events(params)
    if params["doCopy"]:
        print("Copying events...")
        rsync_files(params)
    if params["doMoment"]:
        print("Running moment tensor events...")
        numEventsAddedMoment = moment_events(params)
    if params["doCMT"]:
        print("Running CMT moment tensor events...")
        numEventsAddedCMT = cmt_events(params)

# =============================================================================
#
#                                    MAIN
#
# =============================================================================

if __name__=="__main__":

    run_traveltimes()




import datetime
import os

from flask import Flask
import click
from getpass import getpass
from passlib.apache import HtpasswdFile
from werkzeug.routing import BaseConverter

# Application object
app = Flask(__name__, instance_relative_config=True)
app.config['EXPLAIN_TEMPLATE_LOADING'] = True
app.config['SQLALCHEMY_DATABASE_URI'] = 'postgresql://seismon:seismon@localhost/seismon'
app.config['SQLALCHEMY_BINDS'] = {}

app.config['TEMPLATES_AUTO_RELOAD'] = True

# Turn off memory-intensive modification tracking.
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False

# Default secret key: secure and random. However, sessions are not preserved
# across different Python processes.
app.config['SECRET_KEY'] = os.urandom(24)

# Set 16 MB file size upload limit for posting FITS files.
app.config['MAX_CONTENT_LENGTH'] = 16 * 1024 * 1024

# Celery configuration.
# Use pickle serializer, because it supports byte values.
# Use redis broker, because it supports locks (and thus singleton tasks).
app.config['CELERY_BROKER_URL'] = 'redis://'
app.config['CELERY_ACCEPT_CONTENT'] = ['json', 'pickle']
app.config['CELERY_EVENT_SERIALIZER'] = 'json'
app.config['CELERY_RESULT_SERIALIZER'] = 'pickle'
app.config['CELERY_TASK_SERIALIZER'] = 'pickle'

# Apply instance configuration from application.cfg and application.cfg.d/*.
app.config.from_pyfile('application.cfg', silent=True)
dropin_dir = os.path.join(app.instance_path, 'application.cfg.d')
try:
    dropin_files = os.listdir(dropin_dir)
except (FileNotFoundError, NotADirectoryError):
    pass
else:
    for dropin_file in dropin_files:
        app.config.from_pyfile(os.path.join('application.cfg.d', dropin_file))


class DateTimeConverter(BaseConverter):

    def to_python(self, value):
        try:
            return datetime.datetime.strptime(value, '%y%m%d').date()
        except ValueError:
            try:
                datetime.datetime.strptime(value, '%Y-%m-%d').date()
            except ValueError:
                return datetime.datetime.strptime(value, '%Y-%m-%dT%H:%M:%S')

    def to_url(self, value):
        return value.isoformat(timespec='seconds')


app.url_map.converters['datetime'] = DateTimeConverter


class DateConverter(BaseConverter):

    def to_python(self, value):
        try:
            return datetime.datetime.strptime(value, '%y%m%d').date()
        except ValueError:
            return datetime.datetime.strptime(value, '%Y-%m-%d').date()

    def to_url(self, value):
        return value.isoformat()


app.url_map.converters['date'] = DateConverter
app.jinja_env.filters['zip'] = zip
app.jinja_env.globals['now'] = datetime.datetime.now




# Copyright (C) 2013 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Seismic summary information generator.

This script generates a nested web page and content for
reviewing seismicity at the sites and earthquake status.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""


import os, sys, glob, optparse, shutil, warnings
from operator import itemgetter

if not os.getenv("DISPLAY", None):
    import matplotlib
    matplotlib.use("agg", warn=False)

#import lal.gpstime

import seismon.eqmon

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def parse_commandline():
    """@Parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-p", "--paramsFile", help="Seismon params file.", 
                      default ="/Seismon/Seismon/seismon/input/seismon_params_earthquakesInfo.txt")
    parser.add_option("-e", "--earthquake", help="Earthquake ID.",
                      default ="us10008e3k")
    parser.add_option("-i", "--ifos", help="Ifos.", default="H1,L1,G1,V1,MIT")

    parser.add_option("--eventfilesType", help="Event files type.", default="private")
    parser.add_option("--doEarthquakes",  action="store_true", default=False)

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running pylal_seismon_run..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@Creates seismon params structure
    @param opts
        seismon command line options
    """

    params = seismon.utils.readParamsFromFile(opts.paramsFile)
    params["ifos"] = opts.ifos
    params["earthquake"] = opts.earthquake
    params["eventfilesType"] = opts.eventfilesType
    params["doEarthquakes"] = opts.doEarthquakes

    return params

# =============================================================================
#
#                                    MAIN
#
# =============================================================================

warnings.filterwarnings("ignore")

# Parse command line
opts = parse_commandline()
params = params_struct(opts)

if params["doEarthquakes"]:

    attributeDics = []
    eventfilesLocation = os.path.join(params["eventfilesLocation"],params["eventfilesType"])
    files = glob.glob(os.path.join(eventfilesLocation,"%s*.xml"%params["earthquake"]))

    for numFile in xrange(len(files)):
        file = files[numFile]
        fileSplit = file.replace(".xml","").split("-")
        gps = float(fileSplit[-1])
        attributeDic = seismon.eqmon.read_eqmon(params,file)
        attributeDics.append(attributeDic)

    ifos = params["ifos"].split(",")
    for file,attributeDic in zip(files,attributeDics):
        print file
        for ifo in ifos:
            ifoShort = ifo
            params["ifo"] = ifo
            ifo = seismon.utils.getIfo(params)

            attributeDic = seismon.eqmon.eqmon_loc(attributeDic,ifo)
            traveltimes = attributeDic["traveltimes"][ifo]

            print "%.1f %.1f %.1f %.1f %.1f %.1f %s %.1f"%(attributeDic["GPS"],max(traveltimes["Ptimes"]),max(traveltimes["Stimes"]),max(traveltimes["Rtwotimes"]),max(traveltimes["RthreePointFivetimes"]),max(traveltimes["Rfivetimes"]),ifoShort,attributeDic["SentGPS"])

        print "\n"





# Copyright (C) 2013 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Earthquake xml file generator.

This script generates earthquake xml files using notices from the
internet and USGS PDL client.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""

import os, sys, glob, optparse, warnings, time, json
import numpy as np
import subprocess
from subprocess import Popen
from lxml import etree

import lal.gpstime

from seismon import (eqmon, utils)

import matplotlib
matplotlib.use("AGG")
matplotlib.rcParams.update({'font.size': 18})
from matplotlib import pyplot as plt
from matplotlib import cm

import pymultinest

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def parse_commandline():
    """@parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-i", "--inputFiles", help="Seismon files.")
    parser.add_option("-o", "--outputDirectory", help="output file.")

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running network_eqmon..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@create params structure

    @param opts
        command line options
    """

    params = {}
    params["inputFiles"] = opts.inputFiles
    params["outputDirectory"] = opts.outputDirectory

    return params

def ampRf(M,r,h,Rf0,Rfs,cd,rs):
    #def ampRf(M,r,h,Rf0,Rfs,Q0,Qs,cd,ch,rs):
    # Rf amplitude estimate
    # M = magnitude
    # r = distance [km]
    # h = depth [km]

    # Rf0 = Rf amplitude parameter
    # Rfs = exponent of power law for f-dependent Rf amplitude
    # Q0 = Q-value of Earth for Rf waves at 1Hz
    # Qs = exponent of power law for f-dependent Q
    # cd = speed parameter for surface coupling  [km/s]
    # ch = speed parameter for horizontal propagation  [km/s]
    # rs

    # exp(-2*pi*h.*fc./cd), coupling of source to surface waves
    # exp(-2*pi*r.*fc./ch./Q), dissipation

    fc = 10**(2.3-M/2)
    #Q = Q0/(fc**Qs)
    Af = Rf0/(fc**Rfs)

    #Rf = 1e-3 * M*Af*np.exp(-2*np.pi*h*fc/cd)*np.exp(-2*np.pi*r*(fc/ch)/Q)/(r**(rs))
    Rf = 1e-3 * M*Af*np.exp(-2*np.pi*h*fc/cd)/(r**(rs))

    return Rf

def myprior(cube, ndim, nparams):
    cube[0] = cube[0]*1.0
    cube[1] = cube[1]*2.0
    cube[2] = cube[2]*5000.0
    cube[3] = cube[3]*2.0
    return cube  

def myloglike(cube, ndim, nparams):

    Rf0,Rfs,cd,rs = [cube[i] for i in range(nparams)]
    pred = ampRf(M,r,h,Rf0,Rfs,cd,rs)

    ratio1 = amp/pred
    ratio2 = pred/amp

    ratio = np.max(np.vstack((ratio1,ratio2)).T,axis=1)
    prob = float(len(np.where(ratio < 5.0)[0])) / float(len(ratio))

    if not np.isfinite(prob):
        prob = 0.0
    return prob

def opt_predict(plotDirectory,amp,M,r,h):

    rm_command = "rm %s/2-*"%plotDirectory
    os.system(rm_command)

    # number of dimensions our problem has
    parameters = ["Rf0","Rfs","cd","rs"]
    n_params = len(parameters)

    # we want to see some output while it is running
    progress = pymultinest.ProgressPlotter(n_params = n_params, outputfiles_basename='%s/2-'%plotDirectory); progress.start()
    # run MultiNest
    pymultinest.run(myloglike, myprior, n_params, importance_nested_sampling = False, resume = True, verbose = True, sampling_efficiency = 'model', n_live_points = 1000, outputfiles_basename='%s/2-'%plotDirectory)
    # ok, done. Stop our progress watcher
    progress.stop()

    livefile = os.path.join(plotDirectory,"2-phys_live.points")
    livepoints = np.loadtxt(livefile)
    sens = livepoints[:,4]
    index = np.argmax(sens)
    livepoint = livepoints[index,:]
   
    Rf0 = livepoint[0]
    Rfs = livepoint[1]
    cd = livepoint[2]
    rs = livepoint[3]
 
    return Rf0,Rfs,cd,rs

def predict_events():
    """@write pdl events

    @param params
        seismon params structure
    """

    warnings.filterwarnings("ignore")

    # Parse command line
    opts = parse_commandline()
    params = params_struct(opts)

    if not os.path.isdir(params["outputDirectory"]):
        utils.mkdir(params["outputDirectory"])
    outputFile = os.path.join(params["outputDirectory"],"earthquakes.txt")

    inputFiles = params["inputFiles"].split(",")
    fid = open(outputFile,'w')
    for inputFile in inputFiles:
        events = np.loadtxt(inputFile)
        for ii,event in enumerate(events):
            fid.write("%.1f %.1f %.1f %.1f %.1f %.1f %.1f %.5e %d %d %.1f %.1f %e %.1f %.1f %.1f %.5e %.1f %.5e %.1f %.5e %.1f %d\n"%(event[0],event[1],event[2],event[3],event[4],event[5],event[6],event[7],event[8],event[9],event[10],event[11],event[12],event[13],event[14],event[15],event[16],event[17],event[18],event[19],event[20],event[21],event[22]))
    fid.close()
    events = np.loadtxt(outputFile)

    global M, r, h, amp
    M = events[:,1]
    r = events[:,12]/1000.0
    h = events[:,13]
    amp = events[:,16]
    Rf0 = 0.16
    Rfs = 1.31
    cd = 4672.83
    rs = 0.83
    pred = ampRf(M,r,h,Rf0,Rfs,cd,rs)
    ratio1 = amp/pred
    ratio2 = pred/amp
    ratio = np.max(np.vstack((ratio1,ratio2)).T,axis=1)
    perc = float(len(np.where(ratio > 5.0)[0])) / float(len(ratio))

    Rf0,Rfs,cd,rs = opt_predict(params["outputDirectory"],events[:,16],M,r,h)
    pred = ampRf(M,r,h,Rf0,Rfs,cd,rs)
    events[:,7] = pred
    events = events[events[:,7].argsort()]

    fid = open(outputFile,'w')
    for ii,event in enumerate(events):
        fid.write("%.1f %.1f %.1f %.1f %.1f %.1f %.1f %.5e %d %d %.1f %.1f %e %.1f %.1f %.1f %.5e %.1f %.5e %.1f %.5e %.1f %d\n"%(event[0],event[1],event[2],event[3],event[4],event[5],event[6],event[7],event[8],event[9],event[10],event[11],event[12],event[13],event[14],event[15],event[16],event[17],event[18],event[19],event[20],event[21],event[22]))
    fid.close()
    events = np.loadtxt(outputFile)

    ind = np.arange(len(events[:,7]))
    vmin = 5.0
    vmax = 7.0

    plt.figure()
    ax = plt.gca()
    sc = ax.scatter(ind,events[:,16],s=20,c=events[:,1],vmin=vmin, vmax=vmax,label='Measured')
    ax.set_yscale('log')
    #plt.semilogy(events[:,15],'kx',label='Measured')
    #plt.semilogy(events[:,7],'c*',label='Predicted')
    yerr = np.zeros((2,len(events)))
    yerr[0,:] = events[:,7]*4.0/5.0
    yerr[1,:] = events[:,7]*4.0
    plt.errorbar(ind,events[:,7],yerr=yerr,label='Predicted')
    plt.legend(loc='best',numpoints=1)
    plt.xlabel("Event number")
    plt.ylabel("Ground Velocity [m/s]")
    plt.xlim([-0.5,len(events[:,0])+0.5])
    plt.ylim([1e-8,1e-3])
    cbar = plt.colorbar(sc)
    cbar.set_label("Earthquake Magnitude")
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'prediction.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'prediction.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'prediction.pdf')
    plt.savefig(plotName)
    plt.close()

    vel = (events[:,12]/1000.0)/(events[:,15]-events[:,0])

    distances = np.linspace(0,100000,1000)
    times = distances / 3.5
    plt.figure()
    ax = plt.gca()
    ax.plot(events[:,7],vel,'kx')
    ax.plot([1e-7,1e-3],[3.5,3.5],'k--')
    #ax.set_yscale('log')
    ax.set_xscale('log')
    plt.xlabel("Ground velocity [m/s]")
    plt.ylabel("Earthquake velocity [m/s]")
    plt.xlim([1e-8,1e-3])
    plt.ylim([2,5])
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'velocity.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'velocity.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'velocity.pdf')
    plt.savefig(plotName)
    plt.close()

    vmin = -8
    vmax = -4

    plt.figure()
    ax = plt.gca()
    sc = ax.scatter(events[:,12],events[:,1],c=np.log10(events[:,16]),vmin=vmin, vmax=vmax)
    ax.set_xscale('log')
    plt.xlabel("Distance [m]")
    plt.ylabel("Earthquake Magnitude")
    cbar = plt.colorbar(sc)
    cbar.set_label("log10(Ground velocity [m/s])")
    #plt.xlim([1e-6,1e-3])
    #plt.ylim([5,9])
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'mag_distance.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'mag_distance.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'mag_distance.pdf')
    plt.savefig(plotName)
    plt.close()

    plt.figure()
    ax = plt.gca()
    sc = ax.scatter(events[:,12],events[:,14],c=np.log10(events[:,16]),vmin=vmin, vmax=vmax)
    ax.set_xscale('log')
    plt.xlabel("Distance [m]")
    plt.ylabel("Azimuth [deg]")
    cbar = plt.colorbar(sc)
    cbar.set_label("log10(Ground velocity [m/s])")
    #plt.xlim([1e-6,1e-3])
    #plt.ylim([5,9])
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'azimuth_distance.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'azimuth_distance.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'azimuth_distance.pdf')
    plt.savefig(plotName)
    plt.close()

    plt.figure()
    ax = plt.gca()
    sc = ax.scatter(events[:,14],events[:,1],c=np.log10(events[:,16]),vmin=vmin, vmax=vmax)
    plt.xlabel("Azimuth [deg]")
    plt.ylabel("Earthquake Magnitude")
    cbar = plt.colorbar(sc)
    cbar.set_label("log10(Ground velocity [m/s])")
    #plt.xlim([1e-6,1e-3])
    #plt.ylim([5,9])
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'mag_azimuth.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'mag_azimuth.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'mag_azimuth.pdf')
    plt.savefig(plotName)
    plt.close()

    plt.figure()
    ax = plt.gca()
    indexes = np.where(events[:,22] == 0)[0]
    sc = ax.scatter(events[indexes,12],events[indexes,16],s=20,c='b')
    indexes = np.where(events[:,22] == 1)[0]
    sc = ax.scatter(events[indexes,12],events[indexes,16],s=20,c='g',marker='+')
    indexes = np.where(events[:,22] == 2)[0]
    sc = ax.scatter(events[indexes,12],events[indexes,16],s=20,c='r',marker='x')
    ax.set_xscale('log')
    ax.set_yscale('log')
    plt.xlim([1e5,1e8])
    plt.ylim([8e-7,1e-3])
    plt.xlabel("Distance [m]")
    plt.ylabel("Peak ground motion [m/s]")
    plt.grid()
    plt.show()
    plotName = os.path.join(params["outputDirectory"],'lockloss_vel_distance.png')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'lockloss_vel_distance.eps')
    plt.savefig(plotName)
    plotName = os.path.join(params["outputDirectory"],'lockloss_vel_distance.pdf')
    plt.savefig(plotName)
    plt.close()


# =============================================================================
#
#                                    MAIN
#
# =============================================================================

if __name__=="__main__":

    predict_events()





import os, sys
import numpy as np

import matplotlib
matplotlib.use("AGG")
matplotlib.rcParams.update({'font.size': 18})
from matplotlib import pyplot as plt
from matplotlib import cm

inputFileDirectory="/home/mcoughlin/Seismon/Text_Files/Timeseries/H1_ISI-GND_STS_HAM2_Z_DQ/64/"
#predictionFile="/home/mcoughlin/Seismon/H1/H1O1/1126569617-1136678417/earthquakes/earthquakes.txt"
predictionFile="/home/mcoughlin/Seismon/H1/H1O1/1126073342-1137283217/earthquakes/earthquakes.txt"
outputDirectory="/home/mcoughlin/Seismon/Predictions/H1O1/"

os.system("python seismon_prediction_vs_actual -i %s -p %s -o %s"%(inputFileDirectory,predictionFile,outputDirectory))

inputFileDirectory="/home/mcoughlin/Seismon/Text_Files/Timeseries/L1_ISI-GND_STS_HAM2_Z_DQ/64/"
#predictionFile="/home/mcoughlin/Seismon/L1/L1O1/1126569617-1136678417/earthquakes/earthquakes.txt"
predictionFile="/home/mcoughlin/Seismon/L1/L1O1/1126073342-1137283217/earthquakes/earthquakes.txt"
outputDirectory="/home/mcoughlin/Seismon/Predictions/L1O1/"

os.system("python seismon_prediction_vs_actual -i %s -p %s -o %s"%(inputFileDirectory,predictionFile,outputDirectory))

filenames = ["/home/mcoughlin/Seismon/Predictions/H1S6/earthquakes.txt","/home/mcoughlin/Seismon/Predictions/H1S5/earthquakes.txt","/home/mcoughlin/Seismon/Predictions/H1O1/earthquakes.txt","/home/mcoughlin/Seismon/Predictions/L1S6/earthquakes.txt","/home/mcoughlin/Seismon/Predictions/L1S5/earthquakes.txt","/home/mcoughlin/Seismon/Predictions/L1O1/earthquakes.txt","/home/mcoughlin/Seismon/Predictions/V1VSR4/earthquakes.txt","/home/mcoughlin/Seismon/Predictions/G1GEOHF/earthquakes.txt"]

outputDirectory="/home/mcoughlin/Seismon/Predictions/ALL/"
outputFile = os.path.join(outputDirectory,"earthquakes.txt")

fid = open(outputFile,'w') 
for filename in filenames:
    lines = [line.rstrip() for line in open(filename)]
    for line in lines:
        fid.write("%s\n"%line)
fid.close()

events = np.loadtxt(outputFile)
events = events[events[:,7].argsort()]

plt.figure()
plt.semilogy(events[:,14],'kx',label='Measured')
plt.semilogy(events[:,7],'c*',label='Predicted')
plt.legend(loc=2,numpoints=1)
plt.xlabel("Event number")
plt.ylabel("Ground Velocity [m/s]")
plt.xlim([-0.5,len(events[:,0])+0.5])
plt.ylim([1e-6,1e-3])
plt.show()
plotName = os.path.join(outputDirectory,'prediction.png')
plt.savefig(plotName)
plotName = os.path.join(outputDirectory,'prediction.eps')
plt.savefig(plotName)
plotName = os.path.join(outputDirectory,'prediction.pdf')
plt.savefig(plotName)
plt.close()

vel = (events[:,12]/1000.0)/(events[:,13]-events[:,0])

distances = np.linspace(0,100000,1000)
times = distances / 3.5
plt.figure()
ax = plt.gca()
ax.plot(events[:,7],vel,'kx')
ax.plot([1e-7,1e-3],[3.5,3.5],'k--')
#ax.set_yscale('log')
ax.set_xscale('log')
plt.xlabel("Ground velocity [m/s]")
plt.ylabel("Earthquake velocity [m/s]")
plt.xlim([1e-6,1e-3])
plt.ylim([2,5])
plt.show()
plotName = os.path.join(outputDirectory,'velocity.png')
plt.savefig(plotName)
plotName = os.path.join(outputDirectory,'velocity.eps')
plt.savefig(plotName)
plotName = os.path.join(outputDirectory,'velocity.pdf')
plt.savefig(plotName)
plt.close()

print "Median velocity: %.5f"%np.median(vel)




#!D:\bld\seismon_1650904522807\_h_env\python.exe
#
# code to read the output from seismon_info and write
# the data as epics channels to an EPICS IOC via
# channel access
#
# Data file consists on several lines, one per LIGO site.
# each line is space delimited and has the format
# Earthquake-GPS Magnitude P-wave-arrival S-wave-arrival R3.5-wave-arrival R3.5-wave-arrival R5.0-wave-arrival velocity depth latitude longitude distance ifo
#
# only the line ending with IFO is read
#
# D.Barker LHO 17nov2016
# K. Thorne - use stdenv, allow parameter for data directory

# Setup up CDS standard environment
import sys
sys.path.append('/ligo/cdscfg')
import stdenv as cds
cds.INIT_ENV()

print "Run this for site " + cds.SITE + " ifo " + cds.IFO
# Extract command-line parameters
# 'data directory'
datadir = sys.argv[1:1]
if len(datadir) == 0:
    datadir = "/seisapps/seisdata/all/" + cds.IFO + "O2"

print "seismon data directory is " + datadir

import os
if not os.path.isdir(datadir):
  sys.exit("data directory does not exist")
 
# Make sure it exists

# Setup GPS time
import glob

from gpstime import gpstime
import time
from datetime import datetime
from dateutil import tz

# PYEPICS
from epics import caput

DATADIRPREFIX = "us" # data file names start with this string

# run every two seconds
CYCLE_TIME = 2

counter = 0
uptime = 0

olddatafile = []

while True:
    # find latest file in datadir
  searchpath = datadir + "/*"
  print "searchpath %s" % searchpath
  sys.stdout.flush()
  globstring = glob.iglob(searchpath)
#    print "datadir globstring %s" % globstring
  newest_datadir = max(glob.iglob(searchpath), key=os.path.getctime)
  print "newest_datadir %s" % newest_datadir

# get name os usxxxxxxxx directory under the earthquakes subdirectory
  searchpath = newest_datadir + "/earthquakes/" + DATADIRPREFIX + "*"
  print "searchpath %s" % searchpath
  sys.stdout.flush()

#  First, make sure this is not empty.  If it is, add to uptime, continue
  testglob = glob.glob(searchpath)
  if testglob:

# if OK, get iterator
    globstring = glob.iglob(searchpath)

#    print "datadir globstring %s" % globstring
    newest_usdatadir = max(glob.iglob(searchpath), key=os.path.getctime)
 
    print "newest_usdatadir %s" % newest_usdatadir
    
    datafile = newest_usdatadir + "/earthquakes.txt"

    try:
        df = open(datafile,'r') 
    except IOError:
        print 'cannot open', datafile
    else:
        print "open datafile %s" % datafile
        olddatafile = datafile
        found_ifo_line = False
        for line in df:
            line = line.rstrip()
            if line.endswith(cds.IFO):
	        found_ifo_line = True
	        # decode the IFO line
	        #(eq_gps, eq_mag, p_arr, s_arr, r20_arr, r35_arr, r50_arr, rvel, depth, gps0, gps1, lat, long, dist, ifo) = line.split()
	        (eq_gps, eq_mag, p_arr, s_arr, r20_arr, r35_arr, r50_arr, rvel, gps0, gps1, lat, long, dist, ifo) = line.split()
	        eq_gps_f = float(eq_gps)
	        eq_mag_f = float(eq_mag)
	        p_arr_f = float(p_arr)
	        p_arr_i = int(p_arr_f)
	        s_arr_f = float(s_arr)
	        s_arr_i = int(s_arr_f)
	        r20_arr_f = float(r20_arr)
	        r20_arr_i = int(r20_arr_f)
	        r35_arr_f = float(r35_arr)
	        r35_arr_i = int(r35_arr_f)
	        r50_arr_f = float(r50_arr)
	        r50_arr_i = int(r50_arr_f)
	        rvel_f = float(rvel)
	        #depth_f = float(depth)
	        depth_f = 0.0
	        gps0_d = int(gps0)
	        gps1_d = int(gps1)
	        lat_f = float(lat)
	        long_f = float(long)
	        dist_f = float(dist)
	        #print "EQ_GPS %.1f, EQ_MAG %.1f EQ_LAT %.1f EQ_LONG %.1f EQ_DIST %.1f EQ_DEPTH %.1f PWAVE_TIME %.1f SWAVE_TIME %.1f R2.0_TIME %.1f R3.5_TIME %.1f R5.0_TIME %.1f RVEL %.1f" % \
	        (eq_gps_f, eq_mag_f, lat_f, long_f, dist_f, depth_f, p_arr_f, s_arr_f, r20_arr_f, r35_arr_f, r50_arr_f, rvel_f)
	        
	        # Write data to EPICS Channels, remember ezca appends H1: to channel name
	        caput(cds.IFO + ':CDS-SEISMON_EQ_TIME_GPS', eq_gps_f)
                # get GPS time in UTC, local
                eq_utc = gpstime.fromgps(eq_gps_f)
                eq_utc_str = eq_utc.strftime('%Y-%m-%d %H:%M:%S %Z')
	        caput(cds.IFO + ':CDS-SEISMON_EQ_TIME_UTC', eq_utc_str)               
                eq_ltz = eq_utc.astimezone(tz.tzlocal())
                eq_ltz_str = eq_ltz.strftime('%Y-%m-%d %H:%M:%S %Z') 
	        caput(cds.IFO + ':CDS-SEISMON_EQ_TIME_LTZ', eq_ltz_str)
	        caput(cds.IFO + ':CDS-SEISMON_EQ_MAGNITUDE_MMS', eq_mag_f)
	        caput(cds.IFO + ':CDS-SEISMON_EQ_DEPTH_KM', depth_f)
	        caput(cds.IFO + ':CDS-SEISMON_EQ_LATITUDE_DEG', lat_f)
	        caput(cds.IFO + ':CDS-SEISMON_EQ_LONGITUDE_DEG', long_f)
	        caput(cds.IFO + ':CDS-SEISMON_EQ_' + cds.SITE + '_DISTANCE_M', dist_f)

	        caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_P_ARRIVALTIME_GPS', p_arr_f)   # TESTING
		#p_arr_f = caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_P_ARRIVALTIME_GPS']    # TESTING
	        #p_arr_i = int(p_arr_f)                                 # TESTING
	        caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_S_ARRIVALTIME_GPS', s_arr_f)
		#s_arr_f = caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_S_ARRIVALTIME_GPS']    # TESTING
	        #s_arr_i = int(s_arr_f)                                 # TESTING
	        caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_ARRIVALTIME_GPS', r20_arr_f)
		#r20_arr_f = caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_ARRIVALTIME_GPS']# TESTING
	        #r20_arr_i = int(r20_arr_f)                             # TESTING
	        caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R35_ARRIVALTIME_GPS', r35_arr_f)
		#r35_arr_f = caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R35_ARRIVALTIME_GPS']# TESTING
	        #r35_arr_i = int(r35_arr_f)                             # TESTING
	        caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R50_ARRIVALTIME_GPS', r50_arr_f)
		#r50_arr_f = caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R50_ARRIVALTIME_GPS']# TESTING
	        #r50_arr_i = int(r50_arr_f)                             # TESTING
	        caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_VELOCITY_MPS', rvel_f)
	        caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R35_VELOCITY_MPS', rvel_f)
	        caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R50_VELOCITY_MPS', rvel_f)
    
	        # update program's GPS time
	        gpsnow = int(gpstime.tconvert().gps())
	        caput(cds.IFO + ':CDS-SEISMON_SYSTEM_TIME_GPS', gpsnow)

		# Calculate the time-to-arrive in minutes and seconds
		# P
		p_arrival_time_total_seconds = p_arr_i - gpsnow
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_P_ARRIVALTIME_TOTAL_SECS', p_arrival_time_total_seconds)
		if p_arrival_time_total_seconds < 0:
		    multiplier = -1
		    p_arrival_time_total_seconds = p_arrival_time_total_seconds * -1
                else:
		    multiplier = 1
		p_arrival_time_minutes = int(p_arrival_time_total_seconds)/60
		p_arrival_time_seconds = p_arrival_time_total_seconds%60
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_P_ARRIVALTIME_MINS', multiplier * p_arrival_time_minutes)
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_P_ARRIVALTIME_SECS', multiplier * p_arrival_time_seconds)

		# S
		s_arrival_time_total_seconds = s_arr_i - gpsnow
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_S_ARRIVALTIME_TOTAL_SECS', s_arrival_time_total_seconds)
		if s_arrival_time_total_seconds < 0:
		    multiplier = -1
		    s_arrival_time_total_seconds = s_arrival_time_total_seconds * -1
                else:
		    multiplier = 1
		s_arrival_time_minutes = int(s_arrival_time_total_seconds)/60
		s_arrival_time_seconds = s_arrival_time_total_seconds%60
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_S_ARRIVALTIME_MINS', multiplier * s_arrival_time_minutes)
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_S_ARRIVALTIME_SECS', multiplier * s_arrival_time_seconds)

		# R20
		r20_arrival_time_total_seconds = r20_arr_i - gpsnow
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_ARRIVALTIME_TOTAL_SECS', r20_arrival_time_total_seconds)
		if r20_arrival_time_total_seconds < 0:
		    multiplier = -1
		    r20_arrival_time_total_seconds = r20_arrival_time_total_seconds * -1
                else:
		    multiplier = 1
		r20_arrival_time_minutes = int(r20_arrival_time_total_seconds)/60
		r20_arrival_time_seconds = r20_arrival_time_total_seconds%60
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_ARRIVALTIME_MINS', multiplier * r20_arrival_time_minutes)
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_ARRIVALTIME_SECS', multiplier * r20_arrival_time_seconds)

		# R20
		r20_arrival_time_total_seconds = r20_arr_i - gpsnow
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_ARRIVALTIME_TOTAL_SECS', r20_arrival_time_total_seconds)
		if r20_arrival_time_total_seconds < 0:
		    multiplier = -1
		    r20_arrival_time_total_seconds = r20_arrival_time_total_seconds * -1
                else:
		    multiplier = 1
		r20_arrival_time_minutes = int(r20_arrival_time_total_seconds)/60
		r20_arrival_time_seconds = r20_arrival_time_total_seconds%60
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_ARRIVALTIME_MINS', multiplier * r20_arrival_time_minutes)
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R20_ARRIVALTIME_SECS', multiplier * r20_arrival_time_seconds)

		# 35R
		r35_arrival_time_total_seconds = r35_arr_i - gpsnow
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R35_ARRIVALTIME_TOTAL_SECS', r35_arrival_time_total_seconds)
		if r35_arrival_time_total_seconds < 0:
		    multiplier = -1
		    r35_arrival_time_total_seconds = r35_arrival_time_total_seconds * -1
                else:
		    multiplier = 1
		r35_arrival_time_minutes = int(r35_arrival_time_total_seconds)/60
		r35_arrival_time_seconds = r35_arrival_time_total_seconds%60
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R35_ARRIVALTIME_MINS', multiplier * r35_arrival_time_minutes)
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R35_ARRIVALTIME_SECS', multiplier * r35_arrival_time_seconds)

		# R50
		r50_arrival_time_total_seconds = r50_arr_i - gpsnow
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R50_ARRIVALTIME_TOTAL_SECS', r50_arrival_time_total_seconds)
		if r50_arrival_time_total_seconds < 0:
		    multiplier = -1
		    r50_arrival_time_total_seconds = r50_arrival_time_total_seconds * -1
                else:
		    multiplier = 1
		r50_arrival_time_minutes = int(r50_arrival_time_total_seconds)/60
		r50_arrival_time_seconds = r50_arrival_time_total_seconds%60
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R50_ARRIVALTIME_MINS', multiplier * r50_arrival_time_minutes)
		caput(cds.IFO + ':CDS-SEISMON_' + cds.SITE + '_R50_ARRIVALTIME_SECS', multiplier * r50_arrival_time_seconds)
                break
 
        df.close()
        if not found_ifo_line:
            print "Error - incomplete file. Does not contain a " + cds.IFO + " line"

  counter = counter +1
  uptime = uptime + CYCLE_TIME
  caput(cds.IFO + ':CDS-SEISMON_SYSTEM_UPTIME_SEC', uptime)
  caput(cds.IFO + ':CDS-SEISMON_SYSTEM_COUNTER', counter)
  time.sleep(CYCLE_TIME)





"""
%prog

Michael Coughlin (coughlim@carleton.edu)

This program runs seismon_run.py.

"""

# =============================================================================
#
#                               PREAMBLE
#
# =============================================================================


import os, time
import numpy as np

__author__ = "Michael Coughlin <coughlim@carleton.edu>"
__date__ = "2012/2/7"
__version__ = "0.1"


paramsFile = "/home/mcoughlin/Seismon/seismon/input/seismon_params_L1EQMon.txt"
paramsFile = "/home/mcoughlin/Seismon/seismon/input/seismon_params_L1O1.txt"

gpsStart = 1126051217
gpsEnd = 1133795277

gpsStart = 1126569617
gpsEnd = 1136678417

gpsStart = 1126073342
gpsEnd = 1137283217

segsfile = '/home/mcoughlin/seismon/RfPrediction/data/segs_Locked_L_1126569617_1136649617.txt'
segs = np.loadtxt(segsfile)
 
#os.system("python seismon_run -p %s -s %d -e %d -c L1:ISI-GND_STS_HAM2_Z_DQ --doEarthquakes --doEarthquakesAnalysis --doPSD --eventfilesType iris --minMagnitude 5.0"%(paramsFile,gpsStart,gpsEnd))

for seg in segs:
    gpsStart = int(seg[1]) - 60
    gpsEnd = int(seg[1]) + 60

    os.system("python seismon_run -p %s -s %d -e %d -c L1:ISI-GND_STS_HAM2_Z_DQ,L1:ISI-GND_STS_HAM2_Y_DQ,L1:ISI-GND_STS_HAM2_X_DQ,L1:ISI-GND_STS_HAM5_Z_BLRMS_30M_100M,L1:ISI-GND_STS_HAM5_Y_BLRMS_30M_100M,L1:ISI-GND_STS_HAM5_X_BLRMS_30M_100M --doPSD"%(paramsFile,gpsStart,gpsEnd))

    print "python seismon_run -p %s -s %d -e %d -c L1:ISI-GND_STS_HAM2_Z_DQ,L1:ISI-GND_STS_HAM2_Y_DQ,L1:ISI-GND_STS_HAM2_X_DQ,L1:ISI-GND_STS_HAM5_Z_BLRMS_30M_100M,L1:ISI-GND_STS_HAM5_Y_BLRMS_30M_100M,L1:ISI-GND_STS_HAM5_X_BLRMS_30M_100M --doPSD"%(paramsFile,gpsStart,gpsEnd)
  
#os.system("python seismon_run -p %s -s %d -e %d -c L1:ISI-GND_STS_HAM2_Z_DQ --doEarthquakes --doEarthquakesAnalysis --doPSD --doPlots --eventfilesType iris --minMagnitude 5.0"%(paramsFile,gpsStart,gpsEnd))






# Copyright (C) 2013 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Earthquake xml file generator.

This script generates earthquake xml files using notices from the
internet and USGS PDL client.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""

import os, sys, glob, optparse, warnings, time, json

if not os.getenv("DISPLAY", None):
    import matplotlib
    matplotlib.use("agg", warn=False)

from datetime import datetime
import numpy as np
import subprocess
from subprocess import Popen
from lxml import etree

#import lal.gpstime
import astropy.time

from seismon import (eqmon, utils)

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def parse_commandline():
    """@parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-p", "--paramsFile", help="Seismon params file.",
                      default ="/home/mcoughlin/Seismon/seismon/input/seismon_params_traveltimes.txt")

    parser.add_option("-g", "--gps", help="Earthquake GPS Time.", default=1000000000,type=int)
    parser.add_option("-m", "--magnitude", help="Earthquake magnitude.", default=9.0,type=float)
    parser.add_option("-d", "--depth", help="Earthquake depth [km].", default=50.0,type=float)
    parser.add_option("-l", "--latitude", help="Earthquake Latitude [km].", default=-70.7366,type=float)
    parser.add_option("-n", "--longitude", help="Earthquake Longitude.", default=-30.2407,type=float)

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    parser.add_option("--doFake",  action="store_true", default=False)

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running network_eqmon..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@create params structure

    @param opts
        command line options
    """

    params = utils.readParamsFromFile(opts.paramsFile)
    params["gps"] = opts.gps
    params["magnitude"] = opts.magnitude
    params["depth"] = opts.depth
    params["latitude"] = opts.latitude
    params["longitude"] = opts.longitude

    params["doFake"] = opts.doFake
    params["paramsFile"] = opts.paramsFile

    return params

def write_file(file,tree):
    """@write eqmon file

    @param file
        eqmon file
    @param tree
        xml tree
    """

    f = open(file,'w+')
    f.write('%s'%tree)
    f.close()

def write_info(file,attributeDic):
    """@write eqmon file

    @param file
        eqmon file
    @param attributeDic
        eqmon structure
    """

    root = etree.Element('eqmon')
    for key, value in attributeDic.items():
        if not key == "traveltimes":
            element = etree.SubElement(root,key)
            element.text = str(value)
    #element = etree.SubElement(root,'traveltimes')
    #for key, value in attributeDic["traveltimes"].items():
    #    subelement = etree.SubElement(element,key)
    #    for category in value:
    #        if category in ["Latitudes","Longitudes"]:
    #            continue
    #        subsubelement = etree.SubElement(subelement,category)
    #        subsubelement.text = write_array(value[category])

    
    tree = etree.tostring(root, pretty_print=True)
    write_file(file,tree)
    #tree = etree.ElementTree(root)
    #tree.write(file, pretty_print=True, xml_declaration=True)

def write_array(array):
    """@write array

    @param array
        write array of value
    """
    text = ' '.join([str(x) for x in array])
    return text

def fake_events(params):
    """@write fake events

    @param params
        seismon params structure
    """

    attributeDic = eqmon.fakeeventread(params)
    if not "GPS" in attributeDic:
        return
    if os.path.isfile(os.path.join(params["eventfilesLocation"],"fakepdl/%s/%s/%.0f/eqmon.xml"%(attributeDic["eventName"],attributeDic["eventName"][0:2],attributeDic["GPS"]))):
        return

    file = os.path.join(params["eventfilesLocation"],"fakepdl/%s/%s/%.0f/eqmon.xml"%(attributeDic["eventName"],attributeDic["eventName"][0:2],attributeDic["GPS"]))
            
    utils.mkdir(os.path.join(params["eventfilesLocation"],"fakepdl/%s/%s/%.0f/"%(attributeDic["eventName"],attributeDic["eventName"][0:2],attributeDic["GPS"])))
    write_info(file,attributeDic)

    print "%s added at "%attributeDic["eventName"], time.time(), ". %.3f seconds after event"%(attributeDic["SentGPS"] - attributeDic["GPS"])

def run_traveltimes():
    """@run traveltime calculator
    """

    warnings.filterwarnings("ignore")

    # Parse command line
    opts = parse_commandline()
    # Parse command line
    params = params_struct(opts)

    if params["doFake"]:
        print "Running Fake events..."
        fake_events(params)

# =============================================================================
#
#                                    MAIN
#
# =============================================================================

if __name__=="__main__":

    run_traveltimes()

